import logging
import random
import re
from typing import Optional

from pinhook import plugin  # type: ignore
from pinhook.bot import Bot  # type: ignore

logger = logging.getLogger(__name__)

word_regex = re.compile(r'\W')


def get_that_bread(ctx: Bot.Message) -> str:
    messages_file = ctx.bot.data_path / 'bread_messages.txt'
    try:
        bread_messages = messages_file.read_text().strip().splitlines()
        assert bread_messages
    except (IOError, TypeError, ValueError, AssertionError):
        if not messages_file.exists():
            messages_file.write_text('bread\n')
        logger.warning(f'Bread messages file at {messages_file} not found '
                       'or corrupted, using ["bread"]')
        bread_messages = ['bread']

    return random.choice(bread_messages)


def increment_bread(ctx: Bot.Message) -> None:
    count_file = ctx.bot.data_path / 'breadcount.txt'
    try:
        count = int(count_file.read_text().strip())
    except (IOError, TypeError, ValueError):
        logger.warning(f'Breadcount file at {count_file} not found '
                       'or corrupted, restarting from zero')
        count = 0
    count += 1
    count_file.write_text(str(count))


@plugin.listener('bread')
def bread(ctx: Bot.Message) -> Optional[plugin.Output]:
    words = set(word_regex.split(ctx.text.lower()))
    if 'bread' in words:
        increment_bread(ctx)
    if words == {'bread'}:
        return plugin.message(get_that_bread(ctx))
    return None


@plugin.command(
    '!nukethebread',
    help_text='Reset the breadcount.',
    ops=True,
    ops_msg='This command is restricted to breadpunk admins.'
)
def nukethebread(ctx: Bot.Message) -> plugin.Output:
    (ctx.bot.data_path / 'breadcount.txt').write_text('0')
    return plugin.message('Bread count has been reset.')


@plugin.command(
    '!breadcount',
    help_text='How many times did bakers say bread here?',
)
def breadcount(ctx: Bot.Message) -> plugin.Output:
    count_file = ctx.bot.data_path / 'breadcount.txt'
    try:
        count = int(count_file.read_text().strip())
    except (IOError, TypeError, ValueError) as e:
        return plugin.message(
            'something is wrong with the breadcount! '
            'quick, scream at lucitoast'
        )
        logger.warning('breadcount is broken!', exc_info=e)
    else:
        return plugin.message(
            f'{count} bread{"s" if count != 1 else ""} so far'
        )
